import React from 'react';
import {
  View,
  createElement,
  StyleSheet
} from 'react-native';
import {
  convertBlobToBase64,
  fetchAsBlob,
  resolveAssetDimensions
} from './utils';
const TextAncestorContext = React.createContext(false);

'use strict';

export default class FetchImage extends React.Component {

  constructor(props) {
    super(props);
    this._imageRef = null;
    this.state = {
      layout: null
    };

    const isCached = (typeof props.source === 'string')
      || (props.source.headers === null)
      || Boolean(localStorage.getItem(props.source.uri));
    if (!isCached) {
      this._resolveAssetSource();
    }
  }

  _createLayoutHandler(resizeMode) {
    const { onLayout } = this.props;
    if (resizeMode === 'center' || resizeMode === 'repeat' || onLayout) {
      return (e) => {
        const { layout } = e.nativeEvent;
        onLayout && onLayout(e);
        this.setState(() => ({ layout }));
      };
    }
  };

  _getBackgroundSize(resizeMode) {
    if (this._imageRef && (resizeMode === 'center' || resizeMode === 'repeat')) {
      const { naturalHeight, naturalWidth } = this._imageRef;
      const { height, width } = this.state.layout;
      if (naturalHeight && naturalWidth && height && width) {
        const scaleFactor = Math.min(1, width / naturalWidth, height / naturalHeight);
        const x = Math.ceil(scaleFactor * naturalWidth);
        const y = Math.ceil(scaleFactor * naturalHeight);
        return {
          backgroundSize: `${x}px ${y}px`
        };
      }
    }
  };

  _resolveAssetSource() {
    const source = this.props.source;

    fetchAsBlob(source)
      .then(convertBlobToBase64)
      .then(data => {
        localStorage.setItem(source.uri, data);
        this.forceUpdate();
      })
      .catch(async () => {
        await backOff(
          () => fetchAsBlob(source).then(convertBlobToBase64)
            .then(data => {
              localStorage.setItem(source.uri, data);
              this.forceUpdate();
            })
        );
      });
  }

  _setImageRef(ref) {
    this._imageRef = ref;
  };

  renderImage(hasTextAncestor) {
    const {
      accessibilityLabel,
      accessibilityRole,
      accessible,
      blurRadius,
      defaultSource,
      draggable,
      importantForAccessibility,
      nativeID,
      pointerEvents,
      resizeMode,
      source,
      testID
    } = this.props;

    // This FetchImage component can not contain children.
    if (process.env.NODE_ENV !== 'production') {
      if (this.props.children) {
        throw new Error(
          'The <FetchImage> component cannot contain children. If you want to render content on top of the image, consider using the <ImageBackground> component or absolute positioning.'
        );
      }
    }

    // Load image uri from cache
    let displayImageUri = '' || defaultSource;
    if (typeof source === 'string')
      displayImageUri = source;
    else if (source.headers === null)
      displayImageUri = source.uri;
    else
      displayImageUri = localStorage.getItem(source.uri);

    const backgroundImage = displayImageUri ? `url("${displayImageUri}")` : null;
    const flatStyle = { ...StyleSheet.flatten(this.props.style) };
    const imageSizeStyle = resolveAssetDimensions(source);
    const finalResizeMode = resizeMode || flatStyle.resizeMode || 'cover';

    // Accessibility image allows users to trigger the browser's image context menu
    const hiddenImage = displayImageUri
      ? createElement('img', {
        alt: accessibilityLabel || '',
        draggable: draggable || false,
        ref: (ref) => this._setImageRef(ref),
        src: displayImageUri
      })
      : null;

    return (
      <View
        accessibilityLabel= { accessibilityLabel }
        accessibilityRole = { accessibilityRole }
        accessible = { accessible }
        importantForAccessibility = { importantForAccessibility }
        nativeID = { nativeID }
        onLayout = { this._createLayoutHandler(finalResizeMode) }
        pointerEvents = { pointerEvents }
        style = { [styles.root, hasTextAncestor && styles.inline, imageSizeStyle, flatStyle]}
        testID = { testID }
      >
        <View
          style={
            [
              styles.image,
              resizeModeStyles[finalResizeMode],
              this._getBackgroundSize(finalResizeMode),
              backgroundImage && { backgroundImage },
            ]
          }
        />
        { hiddenImage }
      </View>
    );
  }

  render() {
    return (
      <TextAncestorContext.Consumer>
        {hasTextAncenstor => this.renderImage(hasTextAncenstor)}
      </TextAncestorContext.Consumer>
    );
  }
}

const styles = StyleSheet.create({
  root: {
    flexBasis: 'auto',
    overflow: 'hidden',
    zIndex: 0
  },
  inline: {
    flexDirection: 'row',
    flexWrap: 'wrap',
  },
  image: {
    ...StyleSheet.absoluteFillObject,
    backgroundColor: 'transparent',
    backgroundPosition: 'center',
    backgroundRepeat: 'no-repeat',
    backgroundSize: 'cover',
    height: '100%',
    width: '100%',
    zIndex: -1
  },
  accessibilityImage: {
    ...StyleSheet.absoluteFillObject,
    height: '100%',
    opacity: 0,
    width: '100%',
    zIndex: -1
  }
});

const resizeModeStyles = StyleSheet.create({
  center: {
    backgroundSize: 'auto',
  },
  contain: {
    backgroundSize: 'contain'
  },
  cover: {
    backgroundSize: 'cover'
  },
  none: {
    backgroundPosition: 'center',
    backgroundSize: 'auto'
  },
  repeat: {
    backgroundPosition: '0 0',
    backgroundRepeat: 'repeat',
    backgroundSize: 'auto'
  },
  stretch: {
    backgroundSize: '100% 100%'
  }
});
